package dk.vajhoej.vms.call.script;

import java.io.UnsupportedEncodingException;
import java.util.List;

import static dk.vajhoej.vms.call.lib.VMSCodes.*;

import dk.vajhoej.vms.call.VMSLibrary;
import dk.vajhoej.vms.call.lib.LIBRTL;
import dk.vajhoej.vms.call.lib.LIBRTL_Util;
import dk.vajhoej.vms.call.lib.SYS;

public class DCLLEX extends VMSLibrary {
	/**
	 * DCL lexical F$DELTA_TIME.
	 * @param start start time
	 * @param end end time
	 * @return delta time
	 */
	public static String F$DELTA_TIME(String start, String end) {
		try {
			long tstart = SYS.SYS$BINTIM(start).getResult();
			long tend = SYS.SYS$BINTIM(end).getResult();
			long delta = LIBRTL.LIB$SUB_TIMES(tend,  tstart).getResult();
			return SYS.SYS$ASCTIM(delta).getResult();
		} catch (UnsupportedEncodingException e) {
			return ""; // should never happen
		}
	}
	/**
	 * DCL lexical F$DIRECTORY.
	 * @return current directory
	 */
	public static String F$DIRECTORY() {
		try {
			return SYS.SYS$SETDDIR().getResult();
		} catch (UnsupportedEncodingException e) {
			return ""; // should never happen
		}
	}
	/**
	 * DCL lexical F$EDIT
	 * @param str input string
	 * @param editlist directived
	 * @return output string
	 */
	public static String F$EDIT(String str, String editlist) {
		String res = str;
		String[] dirs = editlist.replace(" ", "").split(",");
		for(String dir : dirs) {
			String ucdir = dir.toUpperCase();
			if(ucdir.equals("COLLAPSE")) {
				res = res.replaceAll("[ \t]", "");
			} else if(ucdir.equals("COMPRESS")) {
				res = res.replaceAll("[ \t]+", " ");
			} else if(ucdir.equals("LOWERCASE")) {
				res = res.toLowerCase();
			} else if(ucdir.equals("TRIM")) {
				res = res.trim();
			} else if(ucdir.equals("UNCOMMENT")) {
				int ix = res.indexOf("!");
				if(ix >= 0) res = res.substring(0, ix);
			} else if(ucdir.equals("UPCASE")) {
				res = res.toUpperCase();
			}
		}
		return res;
	}
	/**
	 * DCL lexical F$ELEMENT.
	 * @param elmno element number
	 * @param delim element delimiter
	 * @param str string
	 * @return specified element
	 */
	public static String F$ELEMENT(int elmno, String delim, String str) {
		return str.split(delim)[elmno];
	}
	/**
	 * DCL lexical F$EXTRACT.
	 * @param start start position
	 * @param len length
	 * @param str string
	 * @return specified sub-string
	 */
	public static String F$EXTRACT(int start, int len, String str) {
		return str.substring(start, start + len);
	}
	/**
	 * DCL lexical F$GETDVI.
	 * @param devnam device name
	 * @param code item code name
	 * @return value
	 */
	public static String F$GETDVI(String devnam, String code) {
		try {
			return LIBRTL_Script.LIB$GETDVI_AsString(code, devnam).getResult();
		} catch (UnsupportedEncodingException e) {
			return ""; // should never happen
		}
	}
	/**
	 * DCL lexical F$GETJPI.
	 * @param pid PID of process
	 * @param code item code name
	 * @return value
	 */
	public static String F$GETJPI(int pid, String code) {
		try {
			return LIBRTL_Script.LIB$GETJPI_AsString(code, pid).getResult();
		} catch (UnsupportedEncodingException e) {
			return ""; // should never happen
		}
	}
	/**
	 * DCL lexical F$INTEGER.
	 * @param str string value
	 * @return integer value
	 */
	public static int F$INTEGER(String str) {
		return Integer.parseInt(str);
	}
	/**
	 * DCL lexical F$LENGTH.
	 * @param str string
	 * @return string length
	 */
	public static int F$LENGTH(String str) {
		return str.length();
	}
	/**
	 * DCL lexical F$LOCATE.
	 * @param substr sub-string
	 * @param str string
	 * @return index of sub-string in string
	 */
	public static int F$LOCATE(String substr, String str) {
		return str.indexOf(substr);
	}
	/**
	 * DCL lexical F$MESSAGE.
	 * @param msgid message id
	 * @return message text
	 */
	public static String F$MESSAGE(int msgid) {
		try {
			return LIBRTL.LIB$SYS_GETMSG(msgid).getResult();
		} catch (UnsupportedEncodingException e) {
			return ""; // should never happen
		}
	}
	/**
	 * DCL lexical F$MODE.
	 * @return mode of process
	 */
	public static String F$MODE() {
		try {
			return LIBRTL_Util.LIB$GETJPI_AsString(JPI$_MODE, 0).getResult();
		} catch (UnsupportedEncodingException e) {
			return ""; // should never happen
		}
	}
	/**
	 * DCL lexical F$PID.
	 * @return PID of current process
	 */
	public static int F$PID() {
		try {
			return LIBRTL_Util.LIB$GETJPI_AsInt(JPI$_PID, 0).getResult();
		} catch (UnsupportedEncodingException e) {
			return 0; // should never happen
		}
	}
	/**
	 * DCL lexical F$PROCESS.
	 * @return process name of current process
	 */
	public static String F$PROCESS() {
		try {
			return LIBRTL_Util.LIB$GETJPI_AsString(JPI$_PRCNAM, 0).getResult();
		} catch (UnsupportedEncodingException e) {
			return ""; // should never happen
		}
	}
	/**
	 * DCL lexical F$SEARCH.
	 * @param filespec file specification
	 * @return matching file name
	 */
	public static String F$SEARCH(String filespec) {
		try {
			List<String> f = LIBRTL_Util.LIB$FIND_FILE(filespec);
			return f.size() > 0 ? f.get(0) : "" ;
		} catch (UnsupportedEncodingException e) {
			return ""; // should never happen
		}
	}
	/**
	 * DCL lexical F$STRING.
	 * @param o integer value
	 * @return string value
	 */
	public static String F$STRING(Object o) {
		return o.toString();
	}
	/**
	 * DCL lexical F$TIME.
	 * @return current time
	 */
	public static String F$TIME() {
		try {
			return SYS.SYS$ASCTIM().getResult();
		} catch (UnsupportedEncodingException e) {
			return ""; // should never happen
		}
	}
	/**
	 * DCL lexical F$TRNLNM.
	 * @param lognam logical name
	 * @return value of logical name
	 */
	public static String F$TRNLNM(String lognam) {
		try {
			return SYS.SYS$TRNLNM("LNM$DCL_LOGICAL", lognam).getResult();
		} catch (UnsupportedEncodingException e) {
			return ""; // should never happen
		}
	}
	/**
	 * DCL lexical F$TRNLNM.
	 * @param lognam logical name
	 * @param tblnam logical table name
	 * @return value of logical name
	 */
	public static String F$TRNLNM(String lognam, String tblnam) {
		try {
			return SYS.SYS$TRNLNM(tblnam, lognam).getResult();
		} catch (UnsupportedEncodingException e) {
			return ""; // should never happen
		}
	}
	/**
	 * DCL lexical F$TYPE.
	 * @param o value
	 * @return type of value
	 */
	public static String F$TYPE(Object o) {
		if(o instanceof Integer) {
			return "INTEGER";
		} else if(o instanceof String) {
			return "String";
		} else {
			return "";
		}
	}
	/**
	 * DCL lexical F$USER.
	 * @return username of current process
	 */
	public static String F$USER() {
		try {
			return LIBRTL_Util.LIB$GETJPI_AsString(JPI$_USERNAME, 0).getResult();
		} catch (UnsupportedEncodingException e) {
			return ""; // should never happen
		}
	}
}
