/*
 * Copyright 2017 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.vms.call.test;

import static org.junit.Assert.*;

import org.junit.Test;

import dk.vajhoej.vms.call.script.DCLLEX;

import static dk.vajhoej.vms.call.lib.VMSStatus.*;

public class TestDCLLEX extends BaseTest {
	@Test
	public void testTime() {
        try {
        	String t1 = DCLLEX.F$TIME();
        	assertVMSCall("F$TIME()", SS$_NORMAL, t1);
        	String t2 = DCLLEX.F$TIME();
        	assertVMSCall("F$TIME()", SS$_NORMAL, t2);
        	String dt = DCLLEX.F$DELTA_TIME(t1, t2);
        	assertVMSCall("F$DELTA_TIME()", SS$_NORMAL, dt);
        } catch(Exception ex) {
        	fail("Unexpected exception: " + ex);
        }
	}
	@Test
	public void testGetInfo() {
        try {
	    	String dir = DCLLEX.F$DIRECTORY();
	    	assertVMSCall("F$DIRECTORY()", SS$_NORMAL, dir);
	    	String mode = DCLLEX.F$MODE();
	    	assertVMSCall("F$MODE()", SS$_NORMAL, mode);
	    	String prcnam = DCLLEX.F$PROCESS();
	    	assertVMSCall("F$PROCESS()", SS$_NORMAL, prcnam);
	    	String usrnam = DCLLEX.F$USER();
	    	assertVMSCall("F$USER()", SS$_NORMAL, usrnam);
        } catch(Exception ex) {
        	fail("Unexpected exception: " + ex);
        }
	}
	@Test
	public void testString() {
        try {
        	String s = "ABC";
        	int slen = DCLLEX.F$LENGTH(s);
	    	assertVMSCall("F$LENGTH()", SS$_NORMAL, slen);
	    	String mid = DCLLEX.F$EXTRACT(2, 1, s);
	    	assertVMSCall("F$EXTRACT()", SS$_NORMAL, mid);
	    	int bpos = DCLLEX.F$LOCATE("B",  s);
	    	assertVMSCall("F$LOCATE()", SS$_NORMAL, bpos);
        } catch(Exception ex) {
        	fail("Unexpected exception: " + ex);
        }
	}
	@Test
	public void testEdit() {
        try {
        	String s = " This is  a test !!!";
	    	assertEquals("F$EDIT COLLAPSE", "Thisisatest!!!", DCLLEX.F$EDIT(s, "COLLAPSE"));
	    	assertEquals("F$EDIT COMPRESS", " This is a test !!!", DCLLEX.F$EDIT(s, "COMPRESS"));
	    	assertEquals("F$EDIT LOWERCASE", " this is  a test !!!", DCLLEX.F$EDIT(s, "LOWERCASE"));
	    	assertEquals("F$EDIT TRIM", "This is  a test !!!", DCLLEX.F$EDIT(s, "TRIM"));
	    	assertEquals("F$EDIT UNCOMMENT", " This is  a test ", DCLLEX.F$EDIT(s, "UNCOMMENT"));
	    	assertEquals("F$EDIT UPCASE", " THIS IS  A TEST !!!", DCLLEX.F$EDIT(s, "UPCASE"));
	    	assertEquals("F$EDIT TRIM,COMPRESS,UPCASE,UNCOMMENT", "THIS IS A TEST ", DCLLEX.F$EDIT(s, "TRIM,COMPRESS,UPCASE,UNCOMMENT"));
        } catch(Exception ex) {
        	fail("Unexpected exception: " + ex);
        }
	}
}
